#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <limits.h>
#include <string.h>
#include "dmm.h"
#define VERSION "Multimeter Version 0.0.6j"
#define TRUE (1==1)
#define FALSE (1==0)

dmm_handle fd;
FILE *outfile; 
char outfilename[255];
int csv=FALSE;

/*****************/

int usage( char *start) {
  printf("%s\n",VERSION);
  printf("Usage: %s [-N <samples>] [-n <print every Nth sample>]\n",start);  
  printf("\t[-T <sample_time>] [-t <print every t secs>] [-o <outfile>] [-v]\n");  
  printf("\t[-i <interface>]  [-s {p|n|u|m|k|M|G|T|<N>}] [-6|9] [-m] [-c] [-C] [-l] [-h]\n\n");
  printf("\t-N X: X values are to be read. (default 1)\n");  
  printf("\t-n X: print every Xth value. (default 1)\n");  
  printf("\t-T X: read values for X seconds.\n");  
  printf("\t-t X: print a value every X seconds.\n");  
  printf("\t-o X: output in file X instead of stdout.\n");  
  printf("\t-C X: output values in CSV format.\n");  
  printf("\t-i X: read from serial interface X (default /dev/ttyS1).\n");  
  printf("\t-s X: displays values scaled X={p|n|u|m|k|M|G|T|<Number>}\n");  
  printf("\t      p .. 10^-12, n .. 10^-9, u .. 10^-6, m .. 10^-3\n");  
  printf("\t      k .. 10^3,   M .. 10^6,  G .. 10^9,  T .. 10^12\n");  
  printf("\t      if X is a Number the unit will be marked with a '*'\n");  
  printf("\t-6  : use bit rate of 600bps (default 1200bps)\n");	
  printf("\t-9  : use bit rate of 9600bps (default 1200bps)\n");	
  printf("\t-m  : mean value is calculated if there are more values.\n");  
  printf("\t-c  : no send command will be sent before reading a\n");  
  printf("\t      value from the multimeter (COMM button pressed).\n");  
  printf("\t-l  : not time limit or sample limit (loop forever)\n");  
  printf("\t-v  : display version number\n");  
  printf("\t-h  : display this screen\n");  
  return -1;
}

int print_value(FILE *outfile, double scale, char scalechar, unsigned long time, double value, char *unit) {

  if (scalechar == ' ') {
      if (csv) {
          fprintf(outfile,"%d,%g,%s\n", time ,value,unit);
      } else { 
          fprintf(outfile,"%6d %g %s\n", time ,value,unit);
      }
  } else if ( strncmp(unit,"Err",3) == 0 ) {
      if (csv) {
          fprintf(outfile,"%d,%g,%s\n", time ,scale*value,unit); 
      } else { 
          fprintf(outfile,"%6d %g %s\n", time ,scale*value,unit); 
      }
  } else { 
      if (csv) {
          fprintf(outfile,"%d,%g,%c%s\n", time ,scale*value,scalechar,unit);
      } else { 
          fprintf(outfile,"%6d %g %c%s\n", time ,scale*value,scalechar,unit);
      }
  }
  if (outfile != stdout)  fflush (outfile);
  return 0;
}

void cleanup(int whatever ) {
  if (outfile != stdout) fclose (outfile);
  dmm_close(fd);
  exit(10);
}
 
void huphandle(int whatever ) {
  if (outfile != stdout) { 
    fclose (outfile);
    outfile=fopen( (char *) outfilename,"a");
  }
} 

/*************** MAIN ***************/

int main(int argc, char *argv[]) {

char port[15]="/dev/ttyS1";  
int portspeed=1200;
char unit[10], *dummy;  
unsigned long i,t,t_prev,sam_n,tM,m=FALSE;
unsigned long samples_N=ULONG_MAX, samples_T=ULONG_MAX;
unsigned long samples_n=ULONG_MAX, samples_t=ULONG_MAX;
int loop=FALSE, send_cmd=TRUE;
char scalechar=' ';
double value, valueM, scale=1.0;
time_t curtime,starttime;

outfile=stdout;
signal(SIGINT, cleanup); 
signal(SIGTERM, cleanup); 
signal(SIGHUP, huphandle);

/******* parse input *******/

for ( i=1; i<argc;i++ ) {

  if (argv[i][0] != '-' ) return usage(argv[0]);
  
  switch (argv[i][1]) {

    case 'v': {
      printf("%s\n",VERSION); return 0; 
    }; break;

    case 'i': {
      if (i+1<argc) {stpncpy( (char *) port, argv[++i],14 ); port[14]='\0';
      } else return usage(argv[0]);
    }; break;

    case 'N': {
      if (i+1<argc) {samples_N=atol( argv[++i]);  } else  return usage(argv[0]);
    }; break;

    case 'n': {
      if (i+1<argc) {samples_n=atol( argv[++i]);  } else  return usage(argv[0]);
    }; break;

    case 'T': {
      if (i+1<argc) {samples_T=atol( argv[++i]);  } else  return usage(argv[0]);
    }; break;

    case 't': {
      if (i+1<argc) {samples_t=atol( argv[++i]);  } else  return usage(argv[0]);
    }; break;

    case '6': { portspeed=600;
    }; break;

    case '9': { portspeed=9600;
    }; break;
    
    case 'm': { m=TRUE; 
    }; break;

    case 'l': { loop=TRUE; 
    }; break;

    case 'c': { send_cmd=FALSE; 
    }; break;

    case 'C': { csv=TRUE; 
    }; break;

    case 'o': {
      if (i+1<argc) {
        stpncpy( (char *) outfilename, argv[++i],254 ); port[254]='\0';
        if ( (outfile=fopen( (char *) outfilename,"w")) == NULL  ) {
          fprintf(stderr,"Error opening file.\n");
          usage(argv[0]); 
        } else {
          curtime = time (NULL);
          fprintf(outfile,"# Generated with %s on %s",VERSION, ctime( &curtime ) );
          fprintf(outfile,"# time[s]  value  unit\n");
        };
      } else return usage(argv[0]);
    }; break; 

    case 's': {
      if (i+1<argc) {
        switch (argv[++i][0]) {
          case 'p': scalechar='p'; scale=1.0e+12; break;
          case 'n': scalechar='n'; scale=1.0e+09; break;
          case 'u': scalechar='u'; scale=1.0e+06; break;
          case 'm': scalechar='m'; scale=1.0e+03; break;
          case 'k': scalechar='k'; scale=1.0e-03; break;
          case 'M': scalechar='M'; scale=1.0e-06; break;
          case 'G': scalechar='G'; scale=1.0e-09; break;
          case 'T': scalechar='T'; scale=1.0e-12; break;
          default:  scalechar='*'; scale=strtod(argv[i] ,&dummy); 
                    if ((scale==0.0) || (dummy==argv[i])) return usage(argv[0]); 
                    break; 
        };       
      } else return usage(argv[0]);
    }; break;

    default: return usage(argv[0]); break;

  }
}

/**********************/

if ( (samples_N==ULONG_MAX) && (samples_T==ULONG_MAX) ) samples_N=1;
if ( (samples_n==ULONG_MAX) && (samples_t==ULONG_MAX) ) samples_n=1;
i=0;t=0; sam_n=0; t_prev=ULONG_MAX;valueM=0.0;tM=0;

fd = dmm_open(port, portspeed);
if ( dmm_success(fd) ) {
  while ( loop || ( (i < samples_N) && (t < samples_T) )) {  

    if (send_cmd) dmm_send(fd);
    value=dmm_read_value( fd, unit);
    if (i==0) starttime = time (NULL);
    curtime = time (NULL); 
    t=curtime-starttime; i++;
    if (strncmp(unit,"Err",3) != 0) {sam_n++;tM+=t; valueM+=value;}

    if  ( ((i%samples_n)==0 ) || (((t%samples_t)==0 ) && (t!=t_prev)) ) { 
    
      if ( m && (i > 1) && (t > 0) && (sam_n > 0) ) {
        print_value(outfile, scale, scalechar, tM/sam_n, valueM/sam_n, unit);
        sam_n=0;valueM=0.0;tM=0;
      } 

      if (! m) {
        print_value(outfile, scale, scalechar, t, value, unit);
      }

    }

    t_prev=t;
  }
}

/*********************/

dmm_close(fd);
if (outfile != stdout) fclose (outfile);
return 0;
}
