#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/termios.h>
#include <sys/ioctl.h>

typedef int dmm_handle;

int dmm_success(dmm_handle dmm) {return (dmm != -1) ;};

dmm_handle dmm_open(const char *device, int speed) {

  int dmm, mdlns;
  struct termios dmm_attr;

  dmm = open(device,O_RDWR | O_NOCTTY);
  if (dmm == -1) {
    fprintf(stderr,"error opening %s: %m\n",device);
    return(dmm);
  }
  
  dmm_attr.c_oflag = 0;
  dmm_attr.c_lflag = 0;
  dmm_attr.c_iflag = IGNBRK | IGNPAR;
  dmm_attr.c_cflag = CS7 | CSTOPB | CREAD | CLOCAL;	
  
  if ( speed == 600 ) {
    cfsetospeed(&dmm_attr,B600); 
    cfsetispeed(&dmm_attr,B600);
  } else if (speed == 1200) {
    cfsetospeed(&dmm_attr,B1200);
    cfsetispeed(&dmm_attr,B1200);
  }
    else 
  {
	cfsetospeed(&dmm_attr,B9600);
	cfsetispeed(&dmm_attr,B9600);
  }
	      

  if ( tcsetattr(dmm,TCSANOW,&dmm_attr) == -1) {  
    fprintf(stderr,"error setting tcsetattr on %s: %m\n",device);
    close(dmm); dmm = -1; return(dmm);
  }

  mdlns = 0;
  if ( ioctl(dmm,TIOCMGET,&mdlns) == -1) {
    fprintf(stderr,"error ioctl TIOCMGET on %s: %m\n",device);
    close(dmm); dmm = -1; return(dmm);
  }

  mdlns &= ~TIOCM_RTS;
  if ( ioctl(dmm,TIOCMSET,&mdlns) == -1) {
    fprintf(stderr,"error ioctl TIOCMSET on %s: %m\n",device);
    close(dmm); dmm = -1; return(dmm);
  }
  
  return(dmm);
};

char * dmm_read(dmm_handle dmm, char *string ) {
  char buffer[14]="              ";   
  char onechar;
  int i=-1,j=0;


  do {
    while(read(dmm, &onechar, 1)<1) {}
    buffer[(++i)%14]=onechar;
  } while ( onechar != '\r'  );

  i++;
  while (j < 13 ) {  
    string[j]=buffer[i%14];
    i++; j++;
  }
  string[13]='\0';

  usleep(500000);
  return string;
}


int dmm_send(dmm_handle dmm) {
  write(dmm, "d\n", 2);
  return 1;
};


int dmm_close(dmm_handle dmm) {
  close (dmm); 
  return 1;
};


double dmm_string_convert(char *string, char *unit) {

  double value=0.0;
  char  *tail;
  int i=0; 
 
  value=strtod(string+2,&tail); 

  if ( tail > (string+6) ) {
 
    tail=string+9; 

    switch ( *tail ) {
      case 'p': value*=1.0E-12; break;
      case 'n': value*=1.0E-9; break;
      case 'u': value*=1.0E-6; break;
      case 'm': value*=1.0E-3; break;
      case 'k': value*=1.0E3; break;
      case 'M': value*=1.0E6; break;
      case 'G': value*=1.0E9; break;
      default: break;
    }
  
    tail++;

    while ( isalpha(*tail ) ) {
      unit[i++]=*tail; tail++;
    }

    if ( isalpha(*string ) && (*string=='A' || *string=='D') ) {

      unit[i++]=' '; unit[i++]='(';

      while ( isalpha(*string ) ) { 
        unit[i++]=*string; string++;
      }
      
      unit[i++]=')';        

    }
    unit[i]='\0';

  } else {
    sprintf(unit,"Err\0");
  }

  return value;
};

double dmm_read_value(dmm_handle dmm, char *unit) {

  char strg[14];

  dmm_read(dmm,strg);
  return dmm_string_convert(strg,unit);   
}
